/************************************************************************
 * obwrapper.cpp OpenBabel wrapper functions
 *
 * Copyright (c) 2004,2005 by Ernst-G. Schmid
 * Copyright (c) 2004,2005 by Bayer Business Services GmbH
 * for explicitly marked functions
 *
 * This file is part of the xchem::tigress project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * lesser GNU General Public License for more details.
 ************************************************************************/
#include <list>

#if HAVE_SSTREAM
#include <sstream>
#elif HAVE_SSTREAM_H
#include <sstream.h>
#endif

#include <mol.h>
#include <fingerprint.h>
#include <obconversion.h>
#include <openbabel/groupcontrib.h>
#include <obiter.h>
#include "obwrapper.h"

using namespace std;
using namespace OpenBabel;
//using namespace XchemTigress;

static const int popcount_counts[] = {
    0,1,1,2,1,2,2,3,1,2,2,3,2,3,3,4,1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,
    1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
    1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
    2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
    1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
    2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
    2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
    3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,4,5,5,6,5,6,6,7,5,6,6,7,6,7,7,8,
};

extern "C" char *
ob_mol_to_smiles (char *molfile)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (molfile);
  string outstring;
  istringstream molstream (tmpStr);
  ostringstream smilesstream;
  char *tmpSmiles;

  conv.SetInAndOutFormats ("MDL", "SMI");
  conv.AddOption ("n", OBConversion::OUTOPTIONS);

  conv.Read (&mol, &molstream);

  if (mol.Empty ())
    return NULL;

  conv.Write (&mol, &smilesstream);

  outstring = smilesstream.str ();

  //outstring = outstring.substr (0, outstring.length ()-1);

  tmpSmiles = strdup (outstring.c_str ());
  
  //printf("Orig: %s\n",tmpSmiles);

  return (tmpSmiles);
}

extern "C" char *
ob_mol_to_canonical_smiles (char *molfile, int iso_and_chiral_markings)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (molfile);
  string outstring;
  istringstream molstream (tmpStr);
  ostringstream smilesstream;
  char *tmpSmiles;

  conv.SetInAndOutFormats ("MDL", "CAN");
  conv.AddOption ("n", OBConversion::OUTOPTIONS);

  if (iso_and_chiral_markings != 0)
    {
      conv.AddOption ("i", OBConversion::OUTOPTIONS);
    }

  conv.Read (&mol, &molstream);

  if (mol.Empty ())
    return NULL;

  conv.Write (&mol, &smilesstream);

  outstring = smilesstream.str ();

  outstring = outstring.substr (0, outstring.length () - 1);

  tmpSmiles = strdup (outstring.c_str ());

  return (tmpSmiles);
}

extern "C" char *
ob_smiles_to_mol (char *smiles)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (smiles);
  string outstring;
  istringstream smilesstream (tmpStr);
  ostringstream molstream;
  char *tmpMolfile;

  conv.SetInAndOutFormats ("SMI", "MDL");

  conv.Read (&mol, &smilesstream);

  if (mol.Empty ())
    return NULL;

  conv.Write (&mol, &molstream);

  outstring = molstream.str ();

  // remove the trailling $$$$\n from the SDFile
  if (outstring.find ("$$$$\n", 0) != string::npos)
    {
      outstring = outstring.substr (0, outstring.length () - 5);
    }
  else if (outstring.find ("$$$$\r\n", 0) != string::npos)
    {
      outstring = outstring.substr (0, outstring.length () - 6);
    }

  tmpMolfile = strdup (outstring.c_str ());

  return (tmpMolfile);
}

extern "C" char *
ob_inchi_to_mol (char *inchi)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (inchi);
  string outstring;
  istringstream inchistream (tmpStr);
  ostringstream molstream;
  char *tmpMolfile;

  conv.SetInAndOutFormats ("INCHI", "MDL");

  conv.Read (&mol, &inchistream);

  if (mol.Empty ())
    return NULL;

  conv.Write (&mol, &molstream);

  outstring = molstream.str ();

  // remove the trailling $$$$\n from the SDFile
  if (outstring.find ("$$$$\n", 0) != string::npos)
    {
      outstring = outstring.substr (0, outstring.length () - 5);
    }
  else if (outstring.find ("$$$$\r\n", 0) != string::npos)
    {
      outstring = outstring.substr (0, outstring.length () - 6);
    }

  tmpMolfile = strdup (outstring.c_str ());

  return (tmpMolfile);
}

extern "C" char *
ob_mol_to_V3000 (char *molfile)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (molfile);
  string outstring;
  istringstream molstream (tmpStr);
  ostringstream V3000stream;
  char *tmpV3000;

  conv.SetInAndOutFormats ("MDL", "MDL");
  conv.AddOption ("2", OBConversion::INOPTIONS);
  conv.AddOption ("3", OBConversion::OUTOPTIONS);

  conv.Read (&mol, &molstream);

  if (mol.Empty ())
    return NULL;

  conv.Write (&mol, &V3000stream);

  outstring = V3000stream.str ();

  // remove the trailling $$$$\n from the SDFile
  if (outstring.find ("$$$$\n", 0) != string::npos)
    {
      outstring = outstring.substr (0, outstring.length () - 5);
    }
  else if (outstring.find ("$$$$\r\n", 0) != string::npos)
    {
      outstring = outstring.substr (0, outstring.length () - 6);
    }

  tmpV3000 = strdup (outstring.c_str ());

  return (tmpV3000);
}

extern "C" char *
ob_V3000_to_mol (char *V3000)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (V3000);
  string outstring;
  istringstream V3000stream (tmpStr);
  ostringstream molstream;
  char *tmpMolfile;

  conv.SetInAndOutFormats ("MDL", "MDL");
  conv.AddOption ("3", OBConversion::INOPTIONS);
  conv.AddOption ("2", OBConversion::OUTOPTIONS);

  conv.Read (&mol, &V3000stream);
  conv.Write (&mol, &molstream);

  outstring = molstream.str ();

  // remove the trailling $$$$\n from the SDFile
  if (outstring.find ("$$$$\n", 0) != string::npos)
    {
      outstring = outstring.substr (0, outstring.length () - 5);
    }
  else if (outstring.find ("$$$$\r\n", 0) != string::npos)
    {
      outstring = outstring.substr (0, outstring.length () - 6);
    }

  tmpMolfile = strdup (outstring.c_str ());

  return (tmpMolfile);
}


/*****************************************************************
 * This function Copyright (c) 2004
 * by Bayer Business Services GmbH
 *****************************************************************/
extern "C" double
ob_molweight (char *smiles)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (smiles);
  istringstream molstream (tmpStr);
  double molweight = 0.0;

  conv.SetInAndOutFormats ("SMI", "SMI");

  conv.Read (&mol, &molstream);

  molweight = mol.GetMolWt ();

  return (molweight);
}

extern "C" char *
ob_hillformula (char *smiles)
{

  string tmpStr (smiles);
  istringstream molstream (tmpStr);
  string molfmla;
  OBMol mol;
  OBConversion conv;
  char *tmpFormula;

  conv.SetInAndOutFormats ("SMI", "SMI");
  conv.Read (&mol, &molstream);

  molfmla = mol.GetFormula ();

  tmpFormula = strdup (molfmla.c_str ());

  return (tmpFormula);
}

extern "C" unsigned int
ob_num_atoms (char *smiles)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (smiles);
  istringstream molstream (tmpStr);
  unsigned int numatoms = 0;

  conv.SetInAndOutFormats ("SMI", "SMI");

  conv.Read (&mol, &molstream);
  //mol.AddHydrogens (false, false);

  numatoms = mol.NumAtoms ();

  return (numatoms);
}

extern "C" unsigned int
ob_num_heavy_atoms (char *smiles)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (smiles);
  istringstream molstream (tmpStr);
  unsigned int numheavyatoms = 0;

  conv.SetInAndOutFormats ("SMI", "SMI");

  conv.Read (&mol, &molstream);

  numheavyatoms = mol.NumHvyAtoms ();

  return (numheavyatoms);
}

extern "C" unsigned int
ob_num_bonds (char *smiles)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (smiles);
  istringstream molstream (tmpStr);
  unsigned int numbonds = 0;

  conv.SetInAndOutFormats ("SMI", "SMI");

  conv.Read (&mol, &molstream);
  //mol.AddHydrogens (false, false);

  numbonds = mol.NumBonds ();

  return (numbonds);
}

extern "C" unsigned int
ob_num_rotatable_bonds (char *smiles)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (smiles);
  istringstream molstream (tmpStr);
  unsigned int numrotors = 0;

  conv.SetInAndOutFormats ("SMI", "SMI");

  conv.Read (&mol, &molstream);

  numrotors = mol.NumRotors ();

  return (numrotors);
}

extern "C" int
ob_total_charge (char *smiles)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (smiles);
  istringstream molstream (tmpStr);
  int totalcharge = 0;

  conv.SetInAndOutFormats ("SMI", "SMI");

  conv.Read (&mol, &molstream);

  totalcharge = mol.GetTotalCharge ();

  return (totalcharge);
}

extern "C" unsigned int
ob_is_nostruct (char *molfile)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (molfile);
  istringstream molstream (tmpStr);

  conv.SetInAndOutFormats ("MDL", "MDL");

  conv.Read (&mol, &molstream);

  return (mol.Empty ())? 1 : 0;
}

extern "C" unsigned int
ob_is_chiral (char *molfile)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (molfile);
  istringstream molstream (tmpStr);
  int chiral = 0;

  conv.SetInAndOutFormats ("MDL", "MDL");

  conv.Read (&mol, &molstream);
  mol.FindChiralCenters ();

  chiral = mol.IsChiral ()? 1 : 0;

  return (chiral);
}

extern "C" int
ob_2D (char *molfile)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (molfile);
  istringstream molstream (tmpStr);
  int twod = 0;

  conv.SetInAndOutFormats ("MDL", "MDL");

  conv.Read (&mol, &molstream);

  if (mol.Has2D ())
    twod++;

  return (twod);
}

extern "C" int
ob_3D (char *molfile)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (molfile);
  istringstream molstream (tmpStr);
  int threed = 0;

  conv.SetInAndOutFormats ("MDL", "MDL");

  conv.Read (&mol, &molstream);

  if (mol.Has3D ())
    threed++;

  return (threed);
}

extern "C" char *
ob_add_hydrogens (char *molfile, int polaronly, int correct4PH)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (molfile);
  string outstring;
  istringstream molstream1 (tmpStr);
  ostringstream molstream2;
  char *tmpMolfile;

  conv.SetInAndOutFormats ("MDL", "MDL");

  conv.Read (&mol, &molstream1);

  mol.AddHydrogens (polaronly != 0, correct4PH != 0);

  conv.Write (&mol, &molstream2);

  outstring = molstream2.str ();

  // remove the trailling $$$$\n from the SDFile
  if (outstring.find ("$$$$\n", 0) != string::npos)
    {
      outstring = outstring.substr (0, outstring.length () - 5);
    }
  else if (outstring.find ("$$$$\r\n", 0) != string::npos)
    {
      outstring = outstring.substr (0, outstring.length () - 6);
    }

  tmpMolfile = strdup (outstring.c_str ());

  return (tmpMolfile);
}

extern "C" char *
ob_delete_hydrogens (char *molfile, int nonpolaronly)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (molfile);
  string outstring;
  istringstream molstream1 (tmpStr);
  ostringstream molstream2;
  char *tmpMolfile;

  conv.SetInAndOutFormats ("MDL", "MDL");

  conv.Read (&mol, &molstream1);

if(mol.NumHvyAtoms () > 0) {
  if (nonpolaronly != 0)
    {
      mol.DeleteNonPolarHydrogens ();
    }
  else
    {
      mol.DeleteHydrogens ();
    }
} else {
    cout << "Warning: Cannot remove hydrogens. Resulting molecule would be empty!" << endl;
}        

  conv.Write (&mol, &molstream2);

  outstring = molstream2.str ();

  // remove the trailling $$$$\n from the SDFile
  if (outstring.find ("$$$$\n", 0) != string::npos)
    {
      outstring = outstring.substr (0, outstring.length () - 5);
    }
  else if (outstring.find ("$$$$\r\n", 0) != string::npos)
    {
      outstring = outstring.substr (0, outstring.length () - 6);
    }

  tmpMolfile = strdup (outstring.c_str ());

  return (tmpMolfile);
}

extern "C" char *
ob_strip_salts (char *molfile, int neutralize_residue)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (molfile);
  string outstring;
  istringstream molstream1 (tmpStr);
  ostringstream molstream2;
  char *tmpMolfile;

  conv.SetInAndOutFormats ("MDL", "MDL");

  conv.Read (&mol, &molstream1);

  mol.StripSalts ();
  
  if(neutralize_residue != 0) {
      FOR_ATOMS_OF_MOL(a, mol) {
         a->SetFormalCharge(0);
         }
  }    

  conv.Write (&mol, &molstream2);

  outstring = molstream2.str ();

  // remove the trailling $$$$\n from the SDFile
  if (outstring.find ("$$$$\n", 0) != string::npos)
    {
      outstring = outstring.substr (0, outstring.length () - 5);
    }
  else if (outstring.find ("$$$$\r\n", 0) != string::npos)
    {
      outstring = outstring.substr (0, outstring.length () - 6);
    }

  tmpMolfile = strdup (outstring.c_str ());

  return (tmpMolfile);
}

/*****************************************************************
 * This function Copyright (c) 2005
 * by Bayer Business Services GmbH
 *****************************************************************/
extern "C" double
ob_exactmass (char *smiles)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (smiles);
  istringstream molstream (tmpStr);
  double exactmass = 0.0;

  conv.SetInAndOutFormats ("SMI", "SMI");

  conv.Read (&mol, &molstream);

  exactmass = mol.GetExactMass ();

  return (exactmass);
}

extern "C" void
ob_fp2 (char *smiles, unsigned int *fp)
{
  OBMol mol;
  OBConversion conv;
  OBFingerprint *fprint;
  string tmpStr (smiles);
  string fp_mode ("FP2");
  istringstream molstream (tmpStr);
  vector < unsigned int >vfp;	
  fprint = OBFingerprint::FindFingerprint (fp_mode);
  vector < unsigned int >::iterator i;
  int j = 0;

  conv.SetInAndOutFormats ("SMI", "SMI");

  conv.Read (&mol, &molstream);

  fprint->GetFingerprint (&mol, vfp);

  /*  for (i = vfp.begin();i != vfp.end();i++) {
     fp[j++]=*i;
     } */
     
  //memset(fp,0x0, FPSIZE2*sizeof(unsigned int));

  memcpy (fp, &vfp[0], FPSIZE2*sizeof(unsigned int));

}

extern "C" void
ob_fp3 (char *smiles, unsigned int *fp)
{
  OBMol mol;
  OBConversion conv;
  OBFingerprint *fprint;
  string tmpStr (smiles);
  //string fp_mode ("evoFP");
  string fp_mode ("FPPC8");
  istringstream molstream (tmpStr);
  vector < unsigned int >vfp;
  fprint = OBFingerprint::FindFingerprint (fp_mode);
  //vector < unsigned int >::iterator i;
  int len;

  conv.SetInAndOutFormats ("SMI", "SMI");

  conv.Read (&mol, &molstream);

  fprint->GetFingerprint (&mol, vfp);

   /* for (i = vfp.begin();i != vfp.end();i++) {
     fp[j++]=*i;
     printf("%i",*i);
     }  */
     
  len = vfp.size();
  
  //cout << len << endl;
  
  if (len > FPSIZE3) {
          len = FPSIZE3;
          cout << "Warning: Index dictionary size exceeded. Only the first " << FPSIZE3*sizeof(unsigned int)*8 << " bits will be used!" << endl; 
          }
          
  memset(fp,0x0, FPSIZE3*sizeof(unsigned int));        

  memcpy (fp, &vfp[0], len*sizeof(unsigned int));

}

//extern "C" void
//ob_fp6 (char *molfile, unsigned int *fp)
//{
//  OBMol mol;
//  OBConversion conv;
//  OBFingerprint *fprint;
//  string tmpStr (molfile);
//  string fp_mode ("FPPC8");
//  istringstream molstream (tmpStr);
//  vector < unsigned int >vfp;
//  fprint = OBFingerprint::FindFingerprint (fp_mode);
//  //vector < unsigned int >::iterator i;
//  int len;
//
//  conv.SetInAndOutFormats ("MDL", "MDL");
//
//  conv.Read (&mol, &molstream);
//
//  fprint->GetFingerprint (&mol, vfp);
//
//   /* for (i = vfp.begin();i != vfp.end();i++) {
//     fp[j++]=*i;
//     printf("%i",*i);
//     }  */
//     
//  len = vfp.size();
//
//  memcpy (fp, &vfp[0], len*sizeof(unsigned int));
//
//}

//extern "C" void
//ob_fp7 (char *molfile, unsigned int *fp)
//{
//  OBMol mol;
//  OBConversion conv;
//  OBFingerprint *fprint;
//  string tmpStr (molfile);
//  string fp_mode ("FP7");
//  istringstream molstream (tmpStr);
//  vector < unsigned int >vfp;	
//  fprint = OBFingerprint::FindFingerprint (fp_mode);
//  vector < unsigned int >::iterator i;
//  int j = 0;
//
//  conv.SetInAndOutFormats ("MDL", "MDL");
//
//  conv.Read (&mol, &molstream);
//
//  fprint->GetFingerprint (&mol, vfp);
//
//  /*  for (i = vfp.begin();i != vfp.end();i++) {
//     fp[j++]=*i;
//     } */
//
//  memcpy (fp, &vfp[0], FPSIZE2*sizeof(unsigned int));
//
//}

extern "C" double
ob_tanimoto (unsigned char *fp1, unsigned char *fp2, unsigned short size)
{
/* Fast tanimoto code with 8 bit LUT by Andrew Dalke as published in http://www.dalkescientific.com/writings/diary/archive/2008/06/27/computing_tanimoto_scores.html.
   Used with permission */

  //double tanimoto = 0.0;
  //const int fpsize = 64; //bytes
  //vector < unsigned int >vfp1 (size), vfp2 (size);	//unsigned int = 4 byte

  //memcpy (&vfp1[0], fp1, size*sizeof(unsigned int));
  //memcpy (&vfp2[0], fp2, size*sizeof(unsigned int));

  //tanimoto = OBFingerprint::Tanimoto (vfp1, vfp2);
  
  int and_count=0, or_count=0, i;
  
  for (i=0; i<size; i++) {
      //cout << (int) (fp2[i] | fp1[i]) << endl;
    or_count  += popcount_counts[fp2[i] | fp1[i]];
    and_count += popcount_counts[fp2[i] & fp1[i]];
  }

  return static_cast<double>(and_count)/or_count;
}

extern "C" unsigned int
ob_popcount (unsigned char *fp, unsigned short size)
{  
  unsigned int popcount=0, i;
  
  for (i=0; i<size; i++) {
    popcount  += popcount_counts[fp[i]];
  }

  return popcount;
}

/* Copyright  The International Union of Pure and Applied Chemistry 2005: IUPAC
  International Chemical Identifier (InChI) (contact: secretariat@iupac.org) */

extern "C" char *
ob_smiles_to_inchi (char *smiles)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (smiles);
  string outstring;
  istringstream molstream (tmpStr);
  ostringstream inchistream;
  char *tmpInChI;

  conv.SetInAndOutFormats ("SMI", "INCHI");
  conv.AddOption ("w", OBConversion::OUTOPTIONS);

  conv.Read (&mol, &molstream);
  conv.Write (&mol, &inchistream);

  //cout << inchistream.str();

  outstring = inchistream.str ();

  outstring = outstring.substr (0, outstring.length () - 1);

  tmpInChI = strdup (outstring.c_str ());

  return (tmpInChI);
}

/*extern "C" unsigned int
ob_SSS_SMARTS (const char *smarts_pattern, char *molfile)
{
  OBMol mol;
  OBConversion conv;
  OBSmartsPattern sp;
  string tmpStr (molfile);
  istringstream molstream (tmpStr);
  vector<vector<int> > maplist;

  conv.SetInAndOutFormats ("MDL", "MDL");

  conv.Read (&mol, &molstream);
  
  if (mol.Empty()) return 0;
  
  sp.Init(smarts_pattern);
  
  //if(mol.NumHvyAtoms() < sp.NumAtoms()) return 0;
  
  sp.Match(mol);

  maplist = sp.GetUMapList();

  return maplist.size();
}*/

extern "C" int
ob_SSS_SMARTS_native (const char *smarts_pattern, char *smiles)
{
  OBMol mol;
  OBConversion conv;
  OBSmartsPattern sp;
  string tmpStr (smiles);
  istringstream molstream (tmpStr);
  //vector <vector<int> > maplist;
  bool match;

  conv.SetInAndOutFormats ("SMI", "SMI");

  conv.Read (&mol, &molstream);

  if (mol.Empty ())
    return 0;

  if (!sp.Init (smarts_pattern)) return -1;;

  if(mol.NumHvyAtoms() < sp.NumAtoms()) return 0;

  match = sp.Match (mol,true);

  //maplist = sp.GetUMapList ();

  //return maplist.size ();
  
  return match ? 1 : 0;
}

extern "C" int
ob_SSS_SMARTS_native_count (const char *smarts_pattern, char *smiles)
{
  OBMol mol;
  OBConversion conv;
  OBSmartsPattern sp;
  string tmpStr (smiles);
  istringstream molstream (tmpStr);
  //vector <vector<int> > maplist;
  unsigned int matchcount = 0;
  

  conv.SetInAndOutFormats ("SMI", "SMI");

  conv.Read (&mol, &molstream);

  if (mol.Empty ())
    return matchcount;

  if(!sp.Init (smarts_pattern)) return -1;

  if(mol.NumHvyAtoms() < sp.NumAtoms()) return 0;

  if (sp.Match(mol,false)) {
      matchcount = sp.GetUMapList().size();
  }    

  //maplist = sp.GetUMapList ();

  //return maplist.size ();
  
  return matchcount;
}

extern "C" double
ob_MR (char *smiles)
{
  OBMol mol;
  OBConversion conv;
  OBMR mr;
  string tmpStr (smiles);
  istringstream molstream (tmpStr);
  double MR = 0.0;

  conv.SetInAndOutFormats ("SMI", "SMI");

  conv.Read (&mol, &molstream);
  //mol.AddHydrogens (false, false);

  MR = mr.Predict (mol);

  return (MR);
}

extern "C" double
ob_PSA (char *smiles)
{
  OBMol mol;
  OBConversion conv;
  OBPSA psa;
  string tmpStr (smiles);
  istringstream molstream (tmpStr);
  double PSA = 0.0;

  conv.SetInAndOutFormats ("SMI", "SMI");

  conv.Read (&mol, &molstream);
  //mol.AddHydrogens (false, false);

  PSA = psa.Predict (mol);

  return (PSA);
}

extern "C" double
ob_logP (char *smiles)
{
  OBMol mol;
  OBConversion conv;
  OBLogP logP;
  string tmpStr (smiles);
  istringstream molstream (tmpStr);
  double LOGP = 0.0;

  conv.SetInAndOutFormats ("SMI", "SMI");

  conv.Read (&mol, &molstream);
  //mol.AddHydrogens (false, false);

  LOGP = logP.Predict (mol);

  return (LOGP);
}

extern "C" unsigned int
ob_num_H_donors (char *smiles)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (smiles);
  istringstream molstream (tmpStr);
  unsigned int numHdonors = 0;

  conv.SetInAndOutFormats ("SMI", "SMI");

  conv.Read (&mol, &molstream);
  
      FOR_ATOMS_OF_MOL(a, mol)
      {
            if(a->IsHbondDonor()) numHdonors++;
      }

  return (numHdonors);
}

extern "C" unsigned int
ob_num_H_acceptors (char *smiles)
{
  OBMol mol;
  OBConversion conv;
  string tmpStr (smiles);
  istringstream molstream (tmpStr);
  unsigned int numHacceptors = 0;

  conv.SetInAndOutFormats ("SMI", "SMI");

  conv.Read (&mol, &molstream);
  
      FOR_ATOMS_OF_MOL(a, mol)
      {
      if(a->IsHbondAcceptor()) numHacceptors++;
      }

  return (numHacceptors);
}
